@@ -1,10 +1,11 @@
-import copy_reg
+from __future__ import absolute_import
+import six.moves.copyreg
 import types
 
 def _pickle_method(method):
-	func_name = method.im_func.__name__
-	obj = method.im_self
-	cls = method.im_class
+	func_name = method.__func__.__name__
+	obj = method.__self__
+	cls = method.__self__.__class__
 	if func_name.startswith('__') and not func_name.endswith('__'): #deal with mangled names
 		cls_name = cls.__name__.lstrip('_')
 		func_name = '_' + cls_name + func_name
@@ -20,5 +21,5 @@ def _unpickle_method(func_name, obj, cls):
 			break
 	return func.__get__(obj, cls)
 
-copy_reg.pickle(types.MethodType, _pickle_method, _unpickle_method)
+six.moves.copyreg.pickle(types.MethodType, _pickle_method, _unpickle_method)
 
