@@ -1,4 +1,5 @@
 
+from __future__ import absolute_import
 import argparse
 import logging
 import os
@@ -115,6 +116,16 @@ def standard_parser():
         dest="plot_suffix"
     )
 
+    parser.add_argument(
+        '--flux-units',
+        help="Flux units.",
+        action="store",
+        type=str,
+        dest="flux_units",
+        choices=['milliJy', 'microJy', 'nanoJy'],
+        default='microJy'
+        )
+
     parser.add_argument(
         '--log-flux',
         help="Plot logarithmic axis for flux.",
@@ -143,6 +154,13 @@ def standard_parser():
         dest="plot_line_labels" 
         )
 
+    parser.add_argument(
+        '--show-line-values',
+        help="Show the line flux values in the output image.",
+        action="store_true", 
+        dest="print_line_values" 
+        )
+
     parser.add_argument(
         '--json-line-labels',
         help="JSON file containing the configuration for labelling emission lines.",
@@ -173,6 +191,13 @@ def standard_parser():
         dest="plot_full_SED" 
         )
 
+    parser.add_argument(
+        '--show-MAP-SED',
+        help="Overplot the full resolution model SED corresponding to the MAP solution.",
+        action="store_true", 
+        dest="plot_MAP_SED" 
+        )
+
     parser.add_argument(
         '--wl-range',
         help="Wavelength range(s) to plot.",
