@@ -1,3 +1,5 @@
+from __future__ import absolute_import
+from __future__ import print_function
 import logging
 import os
 from scipy.interpolate import interp1d
@@ -13,16 +15,18 @@ from astropy.io import ascii
 from astropy.io import fits
 
 import sys
-import dependencies.WeightedKDE as WeightedKDE
-from dependencies.walker_random_sampling import WalkerRandomSampling
+import pyp_beagle.dependencies.WeightedKDE 
+from pyp_beagle.dependencies.walker_random_sampling import WalkerRandomSampling
 
-from beagle_utils import BeagleDirectories, prepare_plot_saving, set_plot_ticks, \
+from .beagle_utils import BeagleDirectories, prepare_plot_saving, set_plot_ticks, \
         prepare_violin_plot, plot_exists, pause, extract_row, is_FITS_file
-from beagle_filters import PhotometricFilters
-from beagle_summary_catalogue import BeagleSummaryCatalogue
-from beagle_residual_photometry import ResidualPhotometry
-from beagle_multinest_catalogue import MultiNestCatalogue
-from beagle_posterior_predictive_checks import PosteriorPredictiveChecks
+from .beagle_filters import PhotometricFilters
+from .beagle_summary_catalogue import BeagleSummaryCatalogue
+from .beagle_residual_photometry import ResidualPhotometry
+from .beagle_multinest_catalogue import MultiNestCatalogue
+from .beagle_posterior_predictive_checks import PosteriorPredictiveChecks
+from .beagle_observed_catalogue import ObservedCatalogue
+from six.moves import range
 
 
 Jy = np.float32(1.E-23)
@@ -34,26 +38,7 @@ p_value_lim = 0.05
 
 
 
-class ObservedCatalogue(object):
-
-    def load(self, file_name):
-
-        """ 
-        Load a photometric catalogue of observed sources. It automatically
-        detects, and loads, FITS or ASCII files depending on the suffix.
-
-        Parameters
-        ----------
-        file_name : str
-            Contains the file name of the catalogue.
-        """
-
-        if is_FITS_file(file_name):
-            self.data = fits.open(file_name)[1].data
-            self.columns = fits.open(file_name)[1].columns
-        else:
-            self.data = ascii.read(file_name, Reader=ascii.basic.CommentedHeader)
-
+class PhotometricCatalogue(ObservedCatalogue):
 
     def extract_fluxes(self, filters, ID, key='ID', aper_corr=1.):
         """ 
@@ -90,6 +75,11 @@ class ObservedCatalogue(object):
 
             # observed flux and its error
             name = filters.data['flux_colName'][j]
+            if not name:
+                flux[j] = -99.
+                flux_err[j] = -99.
+                continue
+
             flux[j] = row[name] * aper_corr * filters.units / Jy
 
             name = filters.data['flux_errcolName'][j]
@@ -105,18 +95,13 @@ class ObservedCatalogue(object):
 
 class Photometry:
 
-    def __init__(self, key='ID', 
-            x_log=False, 
-            log_flux=False,
-            plot_single_solution=None,
-            plot_full_SED=False, 
-            plot_filter_labels=False):
-
+    def __init__(self, **kwargs):
+        
         self.inset_fontsize = BeagleDirectories.inset_fontsize_fraction * BeagleDirectories.fontsize
 
         self.filters = PhotometricFilters()
 
-        self.observed_catalogue = ObservedCatalogue()
+        self.observed_catalogue = PhotometricCatalogue()
 
         self.summary_catalogue = BeagleSummaryCatalogue()
 
@@ -126,20 +111,24 @@ class Photometry:
 
         self.PPC = PosteriorPredictiveChecks()
 
-        self.key = key
+        self.key = kwargs.get('ID_key', 'ID')
         
-        self.x_log = x_log
+        self.x_log = kwargs.get('plot_log_wl', False)
 
-        self.plot_full_SED = plot_full_SED
+        self.plot_full_SED = kwargs.get('plot_full_SED', False)
 
-        self.log_flux = log_flux
+        self.plot_MAP_SED = kwargs.get('plot_MAP_SED', False)
 
-        self.plot_filter_labels = plot_filter_labels
+        self.log_flux = kwargs.get('log_flux', False)
+
+        self.plot_filter_labels = kwargs.get('plot_filter_labels', False)
+
+        self.flux_units = kwargs.get('flux_units', 'microJy')
 
         self.single_solutions = None
-        if plot_single_solution is not None:
+        if kwargs.get('plot_single_solution') is not None:
             self.single_solutions = OrderedDict()
-            with fits.open(plot_single_solution) as f:
+            with fits.open(kwargs.get('plot_single_solution')) as f:
                 self.single_solutions['ID'] = f[1].data['ID']
                 self.single_solutions['row'] = f[1].data['row_index']
 
@@ -174,6 +163,18 @@ class Photometry:
             Whether to redo the plot, even if it already exists
         """
 
+        if self.flux_units == 'milliJy':
+            flux_factor = 1.E+03
+            ylabel = "$f_{\\nu}/\\textnormal{mJy}$"
+        elif self.flux_units == 'microJy':
+            flux_factor = 1.E+06
+            ylabel = "$f_{\\nu}/\\upmu\\textnormal{Jy}$"
+        elif self.flux_units == 'nanoJy':
+            flux_factor = 1.E+09
+            ylabel = "$f_{\\nu}/\\textnormal{nJy}$"
+        else:
+            raise ValueError("Flux units `" + self.flux_units + "` not recognised!")
+
         # Name of the output plot
         plot_name = str(ID)+'_BEAGLE_marginal_SED_phot.pdf'
 
@@ -194,9 +195,8 @@ class Photometry:
 
         # Put observed photometry and its error in arrays
         obs_flux, obs_flux_err = self.observed_catalogue.extract_fluxes(self.filters, ID, key=self.key)
-        obs_flux *= 1.E+09
-        obs_flux_err *= 1.E+09
-
+        obs_flux *= flux_factor
+        obs_flux_err *= flux_factor
 
         fig = plt.figure()
         ax = fig.add_subplot(1, 1, 1)
@@ -245,16 +245,16 @@ class Photometry:
 
         width = 5*np.min(wl_eff[1:]-wl_eff[0:-1])
 
-        kwargs = {'color':'tomato', 'alpha':0.7, 'edgecolor':'black', 'linewidth':0.2}
+        kwargs = {'color':'tomato', 'alpha':0.8, 'edgecolor':'black', 'linewidth':0.2}
 
         for i in range(n_bands):
 
             if old_API:
                 band_name = self.filters.data['label'][sor[i]]
-                xdata = model_sed.data['_'+band_name+'_'] / nanoJy
+                xdata = model_sed.data['_'+band_name+'_'] / Jy * flux_factor
             else:
                 band_name = self.filters.data['name'][sor[i]] + "_APP"
-                xdata = 10.**(0.4*(8.9-model_sed.data[band_name])) / 1.E-09
+                xdata = 10.**(0.4*(8.9-model_sed.data[band_name])) * flux_factor
 
             min_x = np.min(xdata)
             max_x = np.max(xdata)
@@ -281,6 +281,7 @@ class Photometry:
 
         delta_wl = wl_eff[1:]-wl_eff[0:-1]
         delta_wl = np.concatenate(([delta_wl[0]], delta_wl))
+        delta_wl /=  2.
 
         for i in range(n_bands):
 
@@ -312,60 +313,83 @@ class Photometry:
                     color = 'black',
                     marker = "o",
                     markersize = 5,
-                    alpha = 0.7
+                    zorder=3,
+                    alpha = 0.6
                     )
 
 
         # Plot the full SED
-        if 'full sed wl' in hdulist and self.plot_full_SED:
-            wl = hdulist['full sed wl'].data['wl'][0,:]
-            redshifts = hdulist['galaxy properties'].data['redshift']
+        if 'full sed wl' in hdulist: 
 
-            if SED_prob_log_scale:
-                max_prob = np.log10(np.amax(probability))
-                min_prob = np.log10(np.amin(probability))
-            else:
-                max_prob = np.amax(probability)
-                min_prob = np.amin(probability)
+            if self.plot_full_SED or self.plot_MAP_SED:
 
-            indices = np.arange(len(probability))
+                _n_SED_to_plot = 0
+                if self.plot_full_SED: 
+                    _n_SED_to_plot = n_SED_to_plot
 
-            wrand = WalkerRandomSampling(probability, keys=indices)
-            rand_indices = wrand.random(n_SED_to_plot)
-            max_alpha = 0.4
+                if self.plot_MAP_SED:
+                    _n_SED_to_plot += 1
 
-            for j in range(n_SED_to_plot):
+                wl = hdulist['full sed wl'].data['wl'][0,:]
+                redshifts = hdulist['galaxy properties'].data['redshift']
 
-                i = rand_indices[j]
-                SED = hdulist['full sed'].data[i,:]
-
-                z = 0.
-                if redshifts[i] > 0.:
-                    z = redshifts[i]
-
-                # Redshift the SED and wl
-                flux_obs = SED / (1.+z)
-                wl_obs = wl * (1.+z)
-
-                # Convert F_lambda [erg s^-1 cm^-2 A^-1] ----> F_nu [erg s^-1 cm^-2 Hz^-1]
-                flux_obs = (wl_obs)**2/c_light*flux_obs
-
-                # Scale to nanoJy
-                flux_obs = flux_obs * 1.e+23 * 1.e+09
-
-                prob = probability[i]
                 if SED_prob_log_scale:
-                    alpha = (np.log10(prob)-min_prob)/(max_prob-min_prob)
+                    max_prob = np.log10(np.amax(probability))
+                    min_prob = np.log10(np.amin(probability))
                 else:
-                    alpha = (prob-min_prob)/(max_prob-min_prob)
+                    max_prob = np.amax(probability)
+                    min_prob = np.amin(probability)
 
-                alpha=1.
-                ax.plot(wl_obs, 
-                        flux_obs,
-                        color="black",
-                        ls="-",
-                        lw=0.5,
-                        alpha=alpha*max_alpha)
+                indices = np.arange(len(probability))
+
+                wrand = WalkerRandomSampling(probability, keys=indices)
+                rand_indices = wrand.random(n_SED_to_plot)
+
+                for j in range(_n_SED_to_plot):
+
+                    if self.plot_MAP_SED and j == 0:
+                        i = np.argmax(probability)
+                        color = "black"
+                        lw=1.2
+                        max_alpha = 0.8
+                    else:
+                        i = rand_indices[j]
+                        color = "black"
+                        lw=0.5
+                        max_alpha = 0.4
+
+                    SED = hdulist['full sed'].data[i,:]
+
+                    z = 0.
+                    if redshifts[i] > 0.:
+                        z = redshifts[i]
+
+                    # Redshift the SED and wl
+                    flux_obs = SED / (1.+z)
+                    wl_obs = wl * (1.+z)
+
+                    # Convert F_lambda [erg s^-1 cm^-2 A^-1] ----> F_nu [erg s^-1 cm^-2 Hz^-1]
+                    flux_obs = (wl_obs)**2/c_light*flux_obs
+
+                    # Scale to nanoJy
+                    flux_obs = flux_obs / Jy * flux_factor
+
+                    prob = probability[i]
+                    if SED_prob_log_scale:
+                        alpha = (np.log10(prob)-min_prob)/(max_prob-min_prob)
+                    else:
+                        alpha = (prob-min_prob)/(max_prob-min_prob)
+
+                    if self.x_log:
+                        wl_obs = np.log10(wl_obs)
+
+                    alpha=1.
+                    ax.plot(wl_obs, 
+                            flux_obs,
+                            color=color,
+                            ls="-",
+                            lw=lw,
+                            alpha=alpha*max_alpha)
 
 
         # Determine min and max values of y-axis
@@ -427,16 +451,16 @@ class Photometry:
 
         # Define plotting styles
         if self.x_log:
-            ax.set_xlabel("$\\log (\lambda_\\textnormal{eff} / \\textnormal{\AA}$ (observed-frame))")
+            ax.set_xlabel("$\\log (\lambda_\\textnormal{eff} / \\textnormal{\AA})$")
         else:
-            ax.set_xlabel("$\lambda_\\textnormal{eff} / \\textnormal{\AA}$ (observed-frame)")
+            ax.set_xlabel("$\\lambda_\\textnormal{eff} / \\textnormal{\AA}$")
 
-        ax.set_ylabel("$f_{\\nu}/\\textnormal{nanoJy}$")
+        ax.set_ylabel(ylabel)
 
         # Set better location of tick marks
         set_plot_ticks(ax, n_x=5)
 
-        kwargs = {'alpha':0.8}
+        kwargs = {'alpha':0.7}
 
         plt.errorbar(wl_eff[ok], 
                 obs_flux[ok], 
@@ -482,19 +506,19 @@ class Photometry:
 
             # Print the evidence
             try:
-                ax.text(x, y, "$\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=10 )
+                ax.text(x, y, "$\\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=10 )
             except AttributeError:
-                print "ciao"
+                print("ciao")
 
             # Print the average reduced chi-square
             try:
                 row = extract_row(self.PPC.data, ID, key=self.key)
                 aver_chi_square = row['aver_chi_square']
                 y = y1 - (y1-y0)*0.15
-                ax.text(x, y, "$\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=10 )
+                ax.text(x, y, "$\\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=10 )
             except AttributeError:
-                print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                "<chi^2> for the object `" + str(ID) + "` is not available"
+                print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                "<chi^2> for the object `" + str(ID) + "` is not available")
 
             try:
                 row = extract_row(self.PPC.data, ID, key=self.key)
@@ -502,13 +526,13 @@ class Photometry:
                 n_data = row['n_used_bands']
                 y = y1 - (y1-y0)*0.20
                 ax.text(x, y,
-                        "$\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
+                        "$\\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
                         + "{:.2f}".format(aver_red_chi_square) + "\; \
                         (\\textnormal{N}_\\textnormal{data}=" + \
                         "{:d}".format(n_data) + ")" + "$", fontsize=10 )
             except AttributeError:
-                print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                "<chi^2_red> for the object `" + str(ID) + "` is not available"
+                print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                "<chi^2_red> for the object `" + str(ID) + "` is not available")
 
         if y0 < 0.: plt.plot( [x0,x1], [0.,0.], color='gray', lw=1.0 )
 
@@ -553,6 +577,18 @@ class Photometry:
             Whether to redo the plot, even if it already exists
         """
 
+        if self.flux_units == 'milliJy':
+            flux_factor = 1.E+03
+            ylabel = "$f_{\\nu}/\\textnormal{mJy}$"
+        elif self.flux_units == 'microJy':
+            flux_factor = 1.E+06
+            ylabel = "$f_{\\nu}/\\upmu\\textnormal{Jy}$"
+        elif self.flux_units == 'nanoJy':
+            flux_factor = 1.E+09
+            ylabel = "$f_{\\nu}/\\textnormal{nJy}$"
+        else:
+            raise ValueError("Flux units `" + self.flux_units + "` not recognised!")
+
         # Name of the output plot
         plot_name = str(ID)+'_BEAGLE_replic_data_phot.pdf'
 
@@ -588,8 +624,8 @@ class Photometry:
 
         obs_flux, obs_flux_err, wl_eff = obs_flux[sor], obs_flux_err[sor], wl_eff[sor]
 
-        obs_flux *= 1.E+09
-        obs_flux_err *= 1.E+09
+        obs_flux *= flux_factor
+        obs_flux_err *= flux_factor
 
         # Consider only those bands with measurements!
         ok = np.where(obs_flux_err > 0.)[0]
@@ -618,7 +654,7 @@ class Photometry:
         noiseless_flux = np.zeros((n_bands, indices.size))
 
         for i, band_name in enumerate((self.filters.data['label'][sor])):
-            noiseless_flux[i, :] = model_sed.data['_'+band_name+'_'] / nanoJy
+            noiseless_flux[i, :] = model_sed.data['_'+band_name+'_'] / Jy * flux_factor
 
         # Compute the p-value band-by-band
         p_value_bands = np.zeros(n_bands)
@@ -627,16 +663,16 @@ class Photometry:
             
             if obs_flux_err[i] > 0.:
                 obs_discr = (obs_flux[i].repeat(n_replicated)-noiseless_flux[i, :])**2 / obs_flux_err[i].repeat(n_replicated)**2
-                repl_discr = (replic_data.data['_'+band_name+'_']/1.E-09-noiseless_flux[i, :])**2 / obs_flux_err[i].repeat(n_replicated)**2
+                repl_discr = (replic_data.data['_'+band_name+'_']*flux_factor - noiseless_flux[i, :])**2 / obs_flux_err[i].repeat(n_replicated)**2
 
-                replic_fluxes[i,:] = replic_data.data['_'+band_name+'_']/1.E-09
+                replic_fluxes[i,:] = replic_data.data['_'+band_name+'_'] * flux_factor
                 p_value_bands[i] = 1. * np.count_nonzero((repl_discr >
                     obs_discr)) / n_replicated
 
         markers = np.array("o").repeat(n_bands)
         loc = np.where(p_value_bands <= p_value_lim)[0]
         markers[loc] = "o"
-        print "p_value_bands: ", p_value_bands
+        print("p_value_bands: ", p_value_bands)
 
         ext_obs_flux = obs_flux.reshape(n_bands, 1).repeat(n_replicated, 1)
         ext_obs_flux_err = obs_flux_err.reshape(n_bands, 1).repeat(n_replicated, 1)
@@ -647,7 +683,7 @@ class Photometry:
         p_value = 1. * np.count_nonzero((repl_discr >
             obs_discr)) / n_replicated
         
-        print "p_value: ", p_value
+        print("p_value: ", p_value)
 
         median_flux = np.zeros(n_bands)
         pdf_norm = np.zeros(n_bands)
@@ -658,8 +694,8 @@ class Photometry:
         replic_fluxes = np.zeros((n_bands, n_replicated))
         mean_replic_fluxes = np.zeros(n_bands)
         for i, band_name in enumerate((self.filters.data['label'][sor])):
-            mean_replic_fluxes[i] = np.mean(replic_data.data['_'+band_name+'_'])/1.E-09
-            replic_fluxes[i,:] = replic_data.data['_'+band_name+'_']/1.E-09
+            mean_replic_fluxes[i] = np.mean(replic_data.data['_'+band_name+'_']) * flux_factor
+            replic_fluxes[i,:] = replic_data.data['_'+band_name+'_'] * flux_factor
 
         mean_residual = (mean_replic_fluxes-obs_flux)/obs_flux_err 
 
@@ -669,7 +705,7 @@ class Photometry:
                     1).repeat(n_replicated, 1)
 
         residual_covar = np.cov(residual_fluxes)
-        print "residual_covar: ", residual_covar
+        print("residual_covar: ", residual_covar)
 
         # Plot the variance-covariance matrix of residuals
         if 'sns' in sys.modules:
@@ -748,7 +784,7 @@ class Photometry:
 #                cap.set_markeredgewidth(1)
 #
             temp_data = replic_data.data[replic_data_rows[i]]
-            replic_fluxes = np.array(temp_data[1:])/1.E-09
+            replic_fluxes = np.array(temp_data[1:]) * flux_factor
 
             diff_fluxes = (replic_fluxes-obs_flux) / obs_flux_err
 
@@ -848,9 +884,9 @@ class Photometry:
             # Set better location of tick marks
             set_plot_ticks(ax, n_x=4, prune_x='both', prune_y='both')
 
-        xlabel = "$\lambda_\\textnormal{eff} / \\textnormal{\AA}$ (observed-frame)"
+        xlabel = "$\\lambda_\\textnormal{eff} / \\textnormal{\AA}$ (observed-frame)"
         #ylabel = "$f_{\\nu}/\\textnormal{nanoJy}$"
-        ylabel = "$\left(f_{\\nu}^\\textnormal{rep}-f_{\\nu}\\right) / \sigma$"
+        ylabel = "$\\left(f_{\\nu}^\\textnormal{rep}-f_{\\nu}\\right) / \sigma$"
 
         fig.text(0.5, 0.02, xlabel, ha='center', fontsize=fontsize+1)
         fig.text(0.03, 0.5, ylabel, va='center', rotation='vertical', fontsize=fontsize+1)
@@ -868,19 +904,19 @@ class Photometry:
 
             # Print the evidence
             try:
-                ax.text(x, y, "$\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=10 )
+                ax.text(x, y, "$\\log(Z)=" + "{:.2f}".format(self.logEvidence) + "$", fontsize=10 )
             except AttributeError:
-                print "ciao"
+                print("ciao")
 
             # Print the average reduced chi-square
             try:
                 row = extract_row(self.PPC.data, ID, key=self.key)
                 aver_chi_square = row['aver_chi_square']
                 y = y1 - (y1-y0)*0.15
-                ax.text(x, y, "$\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=10 )
+                ax.text(x, y, "$\\langle\chi^2\\rangle=" + "{:.2f}".format(aver_chi_square) + "$", fontsize=10 )
             except AttributeError:
-                print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                "<chi^2> for the object `" + str(ID) + "` is not available"
+                print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                "<chi^2> for the object `" + str(ID) + "` is not available")
 
             try:
                 row = extract_row(self.PPC.data, ID, key=self.key)
@@ -888,13 +924,13 @@ class Photometry:
                 n_data = row['n_used_bands']
                 y = y1 - (y1-y0)*0.20
                 ax.text(x, y,
-                        "$\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
+                        "$\\langle\chi^2/(\\textnormal{N}_\\textnormal{data}-1)\\rangle=" \
                         + "{:.2f}".format(aver_red_chi_square) + "\; \
                         (\\textnormal{N}_\\textnormal{data}=" + \
                         "{:d}".format(n_data) + ")" + "$", fontsize=10 )
             except AttributeError:
-                print "`PosteriorPredictiveChecks` not computed/loaded, hence " \
-                "<chi^2_red> for the object `" + str(ID) + "` is not available"
+                print("`PosteriorPredictiveChecks` not computed/loaded, hence " \
+                "<chi^2_red> for the object `" + str(ID) + "` is not available")
 
 
         #fig.tight_layout()
