@@ -1,18 +1,28 @@
 #!/usr/bin/env python
+from __future__ import absolute_import
 import sys
 import os
 import argparse
 import re
 import numpy as np
-import ConfigParser
+import six.moves.configparser
 import logging
 from matplotlib import rc
 from astropy.io import ascii
 from astropy.io import fits
-
 from pathos.multiprocessing import ProcessingPool 
-from pyp_beagle import *
-from _version import __version__
+
+from .beagle_parsers import standard_parser
+from .beagle_utils import BeagleDirectories, get_files_list
+from .beagle_mock_catalogue import BeagleMockCatalogue
+from .beagle_summary_catalogue import BeagleSummaryCatalogue
+from .beagle_photometry import Photometry
+from .beagle_spectral_indices import SpectralIndices
+from .beagle_spectra import Spectrum
+from .beagle_pdf import PDF
+
+from ._version import __version__
+from six.moves import zip
 
 
 def main():
@@ -29,6 +39,9 @@ def main():
     # Get parsed arguments
     args = parser.parse_args()    
 
+    # Convert argparse.ArgumentParser() Namespace object into a dictionary (see https://stackoverflow.com/a/16878364)
+    args_dict = vars(args)
+
     # Set logging level
     logging.basicConfig(level=args.loglevel)
 
@@ -39,7 +52,7 @@ def main():
     configure_matplotlib()
 
     # Read parameter file
-    config = ConfigParser.SafeConfigParser()
+    config = six.moves.configparser.SafeConfigParser()
 
     # Set fontsize
     BeagleDirectories.fontsize = args.fontsize
@@ -80,6 +93,9 @@ def main():
     # Check if the parameter file contains a PHOTOMETRIC CATALOGUE
     has_photometry = config.has_option('main', 'PHOTOMETRIC CATALOGUE')
 
+    # Check if the parameter file contains a SPECTRAL INDICES CATALOGUE
+    has_spec_indices = config.has_option('main', 'SPECTRAL INDICES CATALOGUE')
+
     # Get list of results files and object IDs from the results directory
     file_list, IDs = get_files_list(suffix=args.suffix)
     if len(file_list) == 0:
@@ -90,17 +106,15 @@ def main():
         #regex = re.compile(r"_MC\w+", re.IGNORECASE)
         regex = re.compile(args.regex_ignore, re.IGNORECASE)
 
-    if args.ID_list is not None:
-        IDs_ = IDs
-        file_list_ = file_list
-        for ID, file in zip(IDs_, file_list_):
-            if regex is not None:
-                ID_ = regex.sub('', ID)
-            else:
-                ID_ = ID
-            if not ID_ in args.ID_list:
+    for i, (ID, file) in enumerate(zip(IDs, file_list)):
+        if args.ID_list is not None:
+            if not ID in args.ID_list:
                 IDs.remove(ID)
                 file_list.remove(file)
+                continue
+        if regex is not None:
+            _ID = regex.sub('', ID)
+            IDs[i] = _ID
 
     # Load mock catalogue
     mock_catalogue = None
@@ -145,13 +159,8 @@ def main():
     if has_photometry:
 
         # Initialize an instance of the main "Photometry" class
-        my_photometry = Photometry(key=args.ID_key, 
-                x_log=args.plot_log_wl, 
-                log_flux=args.plot_log_flux,
-                plot_single_solution=args.plot_single_solution,
-                plot_full_SED=args.plot_full_SED,
-                plot_filter_labels=args.plot_filter_labels)
-
+        my_photometry = Photometry(**args_dict)
+                
         # We can load a set of photometric filters
         try:
             filters_file = os.path.expandvars(config.get('main', 'FILTERS FILE'))
@@ -171,6 +180,21 @@ def main():
         file_name = os.path.expandvars(config.get('main', 'PHOTOMETRIC CATALOGUE'))
         my_photometry.observed_catalogue.load(file_name)
 
+    # ---------------------------------------------------------
+    # --------- Post-processing of spectral indices data -----------
+    # ---------------------------------------------------------
+    if has_spec_indices and args.line_labels_json:
+
+        # Initialize an instance of the main "SpectralIndices" class
+        my_spec_indices = SpectralIndices(**args_dict)
+
+        file_name = os.path.expandvars(config.get('main', 'SPECTRAL INDICES CONFIGURATION'))
+        my_spec_indices.observed_catalogue.configure(file_name)
+
+        # Load observed catalogue
+        file_name = os.path.expandvars(config.get('main', 'SPECTRAL INDICES CATALOGUE'))
+        my_spec_indices.observed_catalogue.load(file_name)
+
     # ---------------------------------------------------------
     # -------- Post-processing of spectroscopic data ----------
     # ---------------------------------------------------------
@@ -178,19 +202,8 @@ def main():
 
         # Initialize an instance of the main "Spectrum" class
         my_spectrum = Spectrum(params_file, 
-                resolution=args.resolution, 
-                plot_full_SED=args.plot_full_SED,
-                wl_range=args.wl_range,
-                line_labels_json=args.line_labels_json,
-                plot_line_labels=args.plot_line_labels, 
-                mock_catalogue=mock_catalogue,
-                log_flux=args.plot_log_flux,
-                show_residual=args.show_residual,
-                print_ID=args.print_ID,
-                wl_rest=args.wl_rest,
-                draw_steps=args.draw_steps,
-                plot_suffix=args.plot_suffix,
-                wl_units=args.wl_units)
+                **args_dict
+                )
 
         my_spectrum.observed_spectrum.configure(config=config)
 
@@ -209,14 +222,15 @@ def main():
         file_names = list()
 
         for ID in IDs:
-            ID_ = ID
+            _ID = ID
             if regex is not None:
-                ID_ = regex.sub('', ID_)
+                _ID = regex.sub('', _ID)
             for line in lines:
-                line_ = trimFitsSuffix(os.path.basename(line))
+                _line = trimFitsSuffix(os.path.basename(line))
                 if regex is not None:
-                    line_ = regex.sub('', line_)
-                if ID_ == line_:
+                    _line = regex.sub('', _line)
+                _line = _line.split('_BEAGLE')[0]
+                if _ID == _line:
                     file_names.append(line)
                     break
 
@@ -232,6 +246,9 @@ def main():
 
             if has_photometry:
                 pool.map(my_photometry.plot_marginal, IDs)
+
+            if has_spec_indices and args.line_labels_json:
+                pool.map(my_spec_indices.plot_line_fluxes, IDs)
         else:
             for i, ID in enumerate(IDs):
                 if has_spectra:
@@ -240,13 +257,16 @@ def main():
                 if has_photometry:
                     my_photometry.plot_marginal(ID)
 
+                if has_spec_indices and args.line_labels_json:
+                    my_spec_indices.plot_line_fluxes(ID)
+
     # Plot the triangle plot
     if args.plot_triangle:
 
         # Set parameter names and labels
         my_PDF = PDF(params_file, 
                 mock_catalogue=mock_catalogue,
-                plot_single_solution=args.plot_single_solution)
+                **args_dict)
 
         if args.n_proc > 1:
             pool.map(my_PDF.plot_triangle, IDs)
